<?php
namespace FM\Service;

use FM\Factory\LocaisAtendimentoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\DataSet;

class LocaisAtendimentoService {

    protected $LocaisAtendimentoFactory;
    protected $Environment;
    protected $newCode;

    public function __construct (EntityManager $entityManager, LocaisAtendimentoFactory $LocaisAtendimentoFactory, Environment $Environment, NewCode $newCode) {
        $this->LocaisAtendimentoFactory  = $LocaisAtendimentoFactory;
        $this->connection                = $entityManager->getConnection();
        $this->Environment               = $Environment;
        $this->newCode                   = $newCode;
    }

    public function getClientes() {
        return $this->LocaisAtendimentoFactory->getClientes();
    }

    public function getLocais() {
        return $this->LocaisAtendimentoFactory->getLocais();
    }

    public function getTiposDeLocais($parametros) {
        
        $param = array(
            "IDTIPOLOCALCLI" => ($parametros['IDTIPOLOCALCLI'] === "0") ? null : $parametros['IDTIPOLOCALCLI']
        );

        return $this->LocaisAtendimentoFactory->getTiposDeLocais($param);
    }

    public function getChecklistPadrao() {
        return $this->LocaisAtendimentoFactory->getChecklistPadrao();
    }

    public function getSetorLocalAtendimento() {
        return $this->LocaisAtendimentoFactory->getSetorLocalAtendimento();
    }

    public function inserirNovoLocal($parametros) {
        $this->connection->beginTransaction();

        try {
            $nrorg      = $this->Environment->getNrorg();
            $nrLocalCliente = $this->newCode->getCodeByTableName('FCM_LOCALCLI', $nrorg, 1, 10, true);
            $cdOperador = $this->Environment->getCdOperador();
            
            $p = array (
                "CDCLIENTE"         => $parametros['CDCLIENTE']
            );

            $cor = $this->LocaisAtendimentoFactory->getCorCliente($p);

            $params = array (
                "NRLOCALCLIENTE"    => $nrLocalCliente,
                "NMLOCALCLIENTE"    => $parametros['NMLOCALCLIENTE'],
                "CDCLIENTE"         => $parametros['CDCLIENTE'],
                "NRTIPOLOCALCLI"    => $parametros['NRTIPOLOCALCLI'],
                "NRLOCALCLIENTEPAI" => $parametros['NRLOCALCLIENTEPAI'],
                "NRCHECKLISTPADR"   => $parametros['NRCHECKLISTPADR'],
                "NRSETOR"           => $parametros['NRSETOR'],
                "DSQRCODE"          => null,
                "NRORG"             => $nrorg,
                "CDOPERINCLUSAO"    => $cdOperador,
                "NRORGINCLUSAO"     => $nrorg,
                "DSCODIGOLOCALCLI"  => $parametros['DSCODIGOLOCALCLI'],
                "DSCORIDLOCAL"      => (!empty($cor)) ? $cor[0]['DSCORIDLOCAL'] : $parametros['DSCORIDLOCAL'],
                "NMABREVLOCALCLI"   => $parametros['NMABREVLOCALCLI']
            );

            $this->LocaisAtendimentoFactory->inserirNovoLocal($params);

            if ($parametros['NRCHECKLISTPADR'] !== null) {
                $nrRelacLocalChk = $this->newCode->getCodeByTableName('FCM_RELACLOCALCHK', $nrorg, 1, 10, true);

                $paramsChk = array(
                    "NRRELACLOCALCHK" => $nrRelacLocalChk,
                    "NRLOCALCLIENTE"  => $nrLocalCliente,
                    "NRCHECKLIST"     => $parametros['NRCHECKLISTPADR']
                );
                
                $this->LocaisAtendimentoFactory->inserirRelacLocalChk($paramsChk);
            }

            $this->connection->commit();

        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getClientesQueAgrupamLocais() {
        return $this->LocaisAtendimentoFactory->getClientesQueAgrupamLocais();
    }

    public function getLocaisAgrupados($parametro) {
        // descomentar essa linha caso seja necessário aplicar o fetchOnDemand como true no frontend
        $param = array(
            "CDCLIENTE" => $parametro['CDCLIENTE'],
            // "NRLOCALCLIENTEPAI" => ($parametro['NRLOCALCLIENTEPAI'] === 'NULL') ? null : $parametro['NRLOCALCLIENTEPAI']
        );

        return $this->LocaisAtendimentoFactory->getLocaisAgrupados($param);
    }

    public function alterarCorCliente($parametros) {
        $this->connection->beginTransaction();

        try {
            $nrorg      = $this->Environment->getNrorg();
            $cdOperador = $this->Environment->getCdOperador();
        
            $param = array(
                "DSCORIDLOCAL" => $parametros['DSCORIDLOCAL'],
                "CDCLIENTE"    => $parametros['CDCLIENTE'],
                "NRORGULTATU"  => $nrorg,
                "CDOPERULTATU" => $cdOperador,

            );
    
            $this->LocaisAtendimentoFactory->alterarCorCliente($param);

            $this->connection->commit();
        }  catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function atualizarLocal($parametros) {
        $this->connection->beginTransaction();

        try {
            $nrorg      = $this->Environment->getNrorg();
            $cdOperador = $this->Environment->getCdOperador();

            $p = array (
                "CDCLIENTE"         => $parametros['CDCLIENTE']
            );

            $cor = $this->LocaisAtendimentoFactory->getCorCliente($p);

            $params = array (
                "NRLOCALCLIENTE"    => $parametros['NRLOCALCLIENTE'],
                "NMLOCALCLIENTE"    => $parametros['NMLOCALCLIENTE'],
                "CDCLIENTE"         => $parametros['CDCLIENTE'],
                "NRTIPOLOCALCLI"    => $parametros['NRTIPOLOCALCLI'],
                "NRLOCALCLIENTEPAI" => $parametros['NRLOCALCLIENTEPAI'],
                "NRCHECKLISTPADR"   => $parametros['NRCHECKLISTPADR'],
                "NMABREVLOCALCLI"   => $parametros['NMABREVLOCALCLI'],
                "NRSETOR"           => $parametros['NRSETOR'],
                "DSCODIGOLOCALCLI"  => $parametros['DSCODIGOLOCALCLI'],
                "NRORGULTATU"       => $nrorg,
                "CDOPERULTATU"      => $cdOperador,
            );
            
            $this->LocaisAtendimentoFactory->atualizarLocal($params);

            $this->connection->commit();

        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function inativarLocais($parametros) {
        $this->connection->beginTransaction();
        
        try {
            $cont = 0;
            $contLocaisFilhos = 0;
            $filtrados = '';
            $nrorg      = $this->Environment->getNrorg();
            $cdOperador = $this->Environment->getCdOperador();

            
            while ($cont <= 2) {
                $indice = (3 - $cont);

                switch ($indice) {
                    case 1:
                        $filtrados = $parametros['TIPO1'];
                        break;
                    case 2:
                        $filtrados = $parametros['TIPO2'];
                        break;
                    case 3:
                        $filtrados = $parametros['TIPO3'];
                        break;
                }

                forEach($filtrados as $filtrado) {
                    // debugApi([$filtrado]);
                    $filhos = $this->LocaisAtendimentoFactory->verificarExistenciaFilhos($filtrado);

                    if ($filhos == 0) {

                        $params = array (
                            "NRLOCALCLIENTE" => $filtrado,
                            "NRORGULTATU"    => $nrorg,
                            "CDOPERULTATU"   => $cdOperador,
                        );
                        
                        $this->LocaisAtendimentoFactory->inativarLocais($params);

                    } else {
                        $contLocaisFilhos++;
                    }
                }
                $cont++;
            }

            $this->connection->commit();
            return $mensagem = ($contLocaisFilhos == 0) ? 'Ok' : 'err';

        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    // --- Funções da TAB DE TIPO DE LOCAL
    public function getTipoDeLocal() {
        return $this->LocaisAtendimentoFactory->getTipoDeLocal();
    }

    public function atualizarTipoLocal($parametros) {
        $this->connection->beginTransaction();

        try {

            $params = array(
                "NRTIPOLOCALCLI" => $parametros['NRTIPOLOCALCLI'],
                "NMTIPOLOCALCLI" => $parametros['NMTIPOLOCALCLI'],
                "IDTIPOLOCALCLI" => $parametros['IDTIPOLOCALCLI'],
                "DSTIPOLOCALCLI" => $parametros['DSTIPOLOCALCLI']
            );
    
            $this->LocaisAtendimentoFactory->atualizarTipoLocal($params);

            $this->connection->commit();

        }  catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function inserirTipoLocal($parametros) {
        $this->connection->beginTransaction();

        try {
            $nrorg      = $this->Environment->getNrorg();
            $nrtipolocalcli = $this->newCode->getCodeByTableName('FCM_TIPOLOCALCLI', $nrorg, 1, 10, true);

            $params = array(
                "NRTIPOLOCALCLI" => $nrtipolocalcli,
                "NMTIPOLOCALCLI" => $parametros['NMTIPOLOCALCLI'],
                "IDTIPOLOCALCLI" => $parametros['IDTIPOLOCALCLI'],
                "DSTIPOLOCALCLI" => $parametros['DSTIPOLOCALCLI'],
                "NRORG"          => $nrorg,
                "IDATIVO"        => "S"
            );
    
            $this->LocaisAtendimentoFactory->inserirTipoLocal($params);

            $this->connection->commit();

        }  catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}